#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>

#ifdef __SYS_UNIX__
#include <sys/socket.h>
#include <netdb.h>
#endif

#ifdef __SYS_WINNT__
#include <winsock.h>
#endif

#define ERROR_PATTERN "****** ERROR ******"

/* Generic method for error */
void fatal(char *msg)
{
	perror(msg);
	exit(-1);
}

/* Print text into stream */
void print(int err, char *str)
{
	if(err == 0)
	{
		fprintf(stderr, "%s", str);
		fflush(stderr);
	}
	else
	{
		fprintf(stdout, "%s", str);
		fflush(stdout);
	}
}

char ret[512] ;

char* parseArg(char * str)
{
	const char sep[] = "\"";
	
	char *spc ;
	char *tok ;

	spc = strchr(str,' ') ;

	memset(ret, '\0', sizeof(ret));

	if(spc != NULL)
		strcat(ret, sep) ;

	if(strchr(str, '"') != NULL){
		tok = (char *)strtok(str, sep);
		do
		{
			strcat(ret, tok) ;
			strcat(ret, "\\\"") ;
		}
		while ((tok = (char *)strtok(NULL, sep)) != NULL);
	}
	else{
		strcat(ret, str);
	}

	if(spc != NULL)
		strcat(ret, sep) ;

	return ret ;
}

/* MAIN */
int main(int argc, char *argv[])
{
	int sock, port;
	struct sockaddr_in sock_addr;
	struct hostent *host;

	const char sep[] = "\n";

	int i, n;
	int retc = 0;	/* return code */
	int frea = 0;	/* first read flag */
	int serr = 1;	/* STDERR flag */
	int brea = 0;	/* nb bytes read from buffer */ 

	/*
	FILE *sockin;
	*/

	char sendbuf[1024];
	char recvbuf[64] ;

	/*char *lin ;*/

	#ifdef __SYS_WINNT__
	WSADATA wsData ; /*  WIN32 */
	#endif

	/* Testing number of arguments */
	if (argc < 3) {
		fprintf(stderr,"Usage: %s host port\n\t list all available services\n", argv[0]);
		fprintf(stderr,"Usage: %s host port serviceName [arg1 arg2 .. argN]\n\t to call a service\n", argv[0]);
		fprintf(stderr,"Usage: %s host port usage serviceName\n\t usage of the service\n", argv[0]);
		fprintf(stderr,"Usage: %s host port help serviceName\n\t help of the service\n", argv[0]);
				
		exit(-1);
	}

	/* Resolving host */
	#ifdef __SYS_WINNT__
	if(WSAStartup(MAKEWORD(1,0), &wsData) != 0)
		fatal("ERROR Socket initialization error");
	#endif

	host = gethostbyname(argv[1]);
	if (host == NULL)
		fatal("ERROR no such host");

	/* Parse port */
	port = atoi(argv[2]);
	if (port < 0)
		fatal("ERROR invalid port number");

	/* Create socket */
	sock = socket(AF_INET, SOCK_STREAM, 0);
	if (sock < 0)
		fatal("ERROR opening socket");

	/* Filling socket informations */
	memset((char *)&sock_addr, '\0', sizeof(sock_addr));
	memcpy((char *)&sock_addr.sin_addr.s_addr, (char *)host->h_addr, host->h_length);
	sock_addr.sin_family = AF_INET;
	sock_addr.sin_port = htons(port);

	/* Connecting to host */
	if (connect(sock,(struct sockaddr*)&sock_addr,sizeof(sock_addr)) < 0)
		fatal("ERROR connecting");

	/* Extract command line */
	memset(sendbuf, '\0', sizeof(sendbuf));
	for (i=3; i<argc; i++) {
		strcat(sendbuf, parseArg(argv[i]));
		if (i < argc-1)
			strcat(sendbuf, " ") ;
	}
	strcat(sendbuf, "\n");

	/* Writing data */
	/*n = write(sock, sendbuf, strlen(sendbuf));*/
	n = send(sock, sendbuf, strlen(sendbuf), 0);
	if (n < 0)
		fatal("ERROR writing to socket");

	memset(sendbuf, '\0', sizeof(sendbuf));

	/* Reading data */
	memset(recvbuf, '\0', sizeof(recvbuf));

	while((n = recv(sock, recvbuf, sizeof(recvbuf), 0)) != 0)
	{
		if(n == -1)
			fatal("ERROR reading from socket");
		
		if(frea == 0)
		{
			frea = 1;
			
			retc = atoi(&recvbuf[0]);
			
			char* temp = (char*)malloc(sizeof(recvbuf));
			strncpy(temp, recvbuf+2, sizeof(recvbuf)-2);
			
			print(retc, temp);
			
			free(temp);
		}
		else
		{
			print(retc, recvbuf) ;
		}

		/*
		brea = 0 ;

		if(recvbuf[0]=='\n')
		{
			print(serr, "\n");
			brea++;
		}
		
		lin = (char *)strtok(recvbuf, sep);
		do
		{
			if(frea == 0)
			{
				retc = atoi(lin);
				frea = 1;
				continue;
			}
			if (strstr(lin, ERROR_PATTERN))
			{
				serr = 0;
				continue;
			}

			print(serr, lin);
			
			brea += strlen(lin); 
			brea++; // for '\n'
			if(brea < n)
				print(serr, "\n");
		}
		while ((lin = (char *)strtok(NULL, sep)) != NULL);
		
		if(recvbuf[n-1]=='\n')
			print(serr, "\n");
		*/
			
		memset(recvbuf, '\0', sizeof(recvbuf));
	}
	/* UNIX
	sockin = fdopen(sock, "r");
	while (fgets(recvbuf, sizeof(recvbuf), sockin)) {
		if (frea == 0) {
			retc = atoi(recvbuf);
			frea = 1;
			continue;
		}
		if (strstr(recvbuf, ERROR_PATTERN)) {
			serr = 0;
			continue;
		}
		if (serr == 0) fprintf(stderr, "%s", recvbuf);
		else           fprintf(stdout, "%s", recvbuf);

		memset(recvbuf, '\0', sizeof(recvbuf));
	}
	*/

	/* Close connection */
	#ifdef __SYS_UNIX__
	/*fclose(sockin);*/
	close(sock);
	#endif

	#ifdef __SYS_WINNT__
	closesocket(sock);
	#endif

	return retc;
}

